from flask import jsonify, request
from datetime import date
from ..policy import Policy
from ..models import Activity, ActivityType
from ..utils import get_policy_class
from . import bp


@bp.route('/policy/<string:policy_number>/<string:effective_date>')
@bp.route('/policy/<string:policy_number>')
def get_policy(policy_number, effective_date=None):
    #
    # fetches a Policy by policy_number & effective_data
    # and returns it
    #

    # set default effective_date if needed
    if effective_date is None:
        effective_date = str(date.today())

    try:
        # get Policy
        policy = get_policy_class()(policy_number, effective_date)
        if policy.fetch():
            return jsonify(policy.get()), 200
    except Exception as e:
        return jsonify({'error': str(e)}), 400

    return jsonify({'error': 'Policy not found'}), 404


@bp.route(f'/activity', methods=['POST'])
def new_activity():
    #
    # create new activity
    #

    # get post data
    data = request.get_json()

    # create activity
    try:
        activity = Activity.create_from_json(data)
        return jsonify({
            'id': str(activity),
            'status': 'accepted',
            'msg': 'Activity accepted',
        }), 202
    except Exception as e:
        print(e)
        return jsonify({'error': 'Bad Request'}), 400



#
# emulator of a Policy Management System
#
import json
from time import sleep

DELAY_SECONDS = 1
POLICY_DATASET = 'policy_system/data/policies.json'

def get(policy_number, effective_date):
    #
    # returns policy from dataset
    #

    # load policies
    with open(POLICY_DATASET, 'r') as f:
        policies = json.load(f)
        # delay emulation
        sleep(DELAY_SECONDS)
        request_number = policy_number.upper().replace('-', '')
        request_date = effective_date.replace('-', '')
        print(f'**** REQUEST POLICY: {request_number} {request_date}')
        for item in policies:
            if request_number == item['number'].replace('-', '') and request_date == item['effective_date'].replace('-', ''):
                return item


from datetime import datetime


class Policy:
    #
    # Policy details, activities, attributes 
    #

    # policy statuses with corresponded possible activities 
    activities_by_state = {}

    # policy attribute descriptions
    attributes_policy = {}

    # product line attribute descriptions
    attributes_product_line = {}

    # insured person attribute descriptions
    attributes_insured_person = {}

    # insured object attribute descriptions
    attributes_insured_object = {}

    # insured object type attribute descriptions
    attributes_insured_object_type = {}

    # implementation attribute descriptions
    attributes_implementation = {}

    def __init__(self, number, date):
        #
        # initialization
        #

        self.number = number
        self.effective_date = date
        self.data = None

    def fetch(self):
        #
        # IMPORTANT: this method should be define within custon implementation
        #
        # fetches policy details from Policy Management System
        #

        raise Exception('Fetch method method is not set in policy class')

    
    def get_status(self):
        #
        # returns status of policy
        #
        if self.data:
            return self.data.get('status')

    def get_product_line(self):
        #
        # returns name of policy's product line
        #

        if self.data and self.data.get('product_line'):
            return self.data.prosuct_line.get('name')

    def get_insured_is_person(self):
        #
        # returns:
        # - True if insured person
        # - False if insured object
        # - None otherwise
        #

        if self.data and self.data.get('insured_object'):
            return self.data['insured_object'].get('is_person')

    def get_insured_object_type(self):
        #
        # returns type of insure object
        #

        if self.get_insured_is_person() is False:
            return self.data['insured_object'].get('type')


    def get_possible_activities(self):
        #
        # returns list of possible activitites for policy
        #

        return self.activities_by_state.get(self.get_status())

    def get_attribute_descriptions(self):
        #
        # returns name-description of attributes applicable to policy 
        #

        return {
            'policy': self.attributes_policy,
            'product_line': self.attributes_product_line.get(self.data['product_line'].get('name')),
            'insured_object': self.attributes_insured_person if self.get_insured_is_person() else self.attributes_insured_object,
            'insured_object_type': self.attributes_insured_object_type.get(self.get_insured_object_type()),
            'implementation': self.attributes_implementation,
        }

    def get(self):
        #
        # returns Policy data
        #

        if self.data:
            return {
                'policy': self.data,
                'possible_activities': self.get_possible_activities(),
                'attributes': self.get_attribute_descriptions(),
            }


